Le design pattern des additions et soustractions d'attribut en python
=====================================================================
![category](développement)
![tag](designpattern)
![tag](python)

méthode simple mais tout les attributs sont testés

```python
class C(object):

    def __init__(self):
        self.x = 0
        self.y = 0

    def __setattr__(self, name, value):
        """Ne modifie pas x si la valeur est négative"""
        if name == 'x' and value < 0:
            return
        # Pour tout autre attribut on affecte la valeur
        object.__setattr__(self, name, value)
```

méthode plus fine

```python
class UserList(object):
    def __init__(self, val=""):
        self._val = val

    def __add__(self, x):
        if len(self._val) > 0:
            return UserList(self._val + ',' + x)
        return UserList(x)

    def __iadd__(self, x):
        if len(self._val) > 0:
            self._val = self._val + ',' + x
        else:
            self._val = x
        return self

    def __sub__(self, x):
        return UserList(','.join([item for item in self._val.split(',') if item != x]))

    def __isub__(self, x):
        self._val = ','.join([item for item in self._val.split(',') if item != x])
        return self

    def __str__(self):
        return str(self._val)

    def __repr__(self):
        return str(self._val)

class Mail(object):
    def __init__(self, server="", subject="", to="", cc=None, dest=None):
        self._cc = UserList()
        self._for = UserList()
        self.server = server
        self.subject = subject
        self.msg = ""
        self.to = to

        if cc != None: self.cc += cc
        if dest != None: self.dest += dest

    def getcc(self):
        return self._cc

    def setcc(self, val):
        if not isinstance(val, UserList):
            val = UserList(val)
        self._cc = val

    def getdest(self):
        return self._for

    def setdest(self, val):
        if not isinstance(val, UserList):
            val = UserList(val)
        self._for = val

    cc = property(getcc, setcc)
    dest = property(getdest, setdest)

mail = Mail(server='srvpzex1',
             subject='test',
             to='informatiquepz@proplast-group.com',
             cc='f.bourel@proplast-group.com',
             dest='a.billard@proplast-group.com')

mail.cc += 'f.gaborit@proplast-group.com'
mail.cc = mail.cc + 'f.aoustin@proplast-group.com'
mail.dest += 'appro3@proplast-group.com'
print mail.cc
print mail.dest
mail.cc -= 'f.bourel@proplast-group.com'
mail.cc = mail.cc - 'f.aoustin@proplast-group.com'
print mail.cc```
